/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * $Id: JMSMessageListener.java 5966 2010-01-19 09:42:37Z alitokmen $
 */
package org.ow2.jasmine.monitoring.eos.notification.service.jms;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;

import org.ow2.jasmine.event.beans.JasmineEventNotification;
import org.ow2.jasmine.event.beans.JasmineEventNotificationDetail;
import org.ow2.jasmine.event.beans.JasmineEventNotificationExecutedAction;
import org.ow2.jasmine.monitoring.eos.common.EventManager;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * @author Jean-Pierre Poutcheu
 */
public class JMSMessageListener implements MessageListener, Serializable {

    /**
     * Logger.
     */
   private static Log logger = LogFactory.getLog(JMSMessageListener.class);

    /**
     * The notification events manager.
     */
    private EventManager<JasmineEventNotification> manager = null;

    /**
     * Constructor of the Message Listener. initialize the manager.
     * @param evManager the event manager
     */
    public JMSMessageListener(final EventManager<JasmineEventNotification> evManager) {
        super();
        this.manager = evManager;
    }

    /**
     * Insert the message event into the manager.
     *
     * @param message the notification event message
     */
    public void onMessage(final Message message) {
        logger.debug("onMessage", message);

        try {
            if (message instanceof ObjectMessage) {

                ObjectMessage om = (ObjectMessage) message;
                Object jasmineEventNotificationObject = om.getObject();

                if (jasmineEventNotificationObject != null &&
                    jasmineEventNotificationObject instanceof JasmineEventNotification) {

                    // TODO: why are we copying the object anyway ?
                    JasmineEventNotification notification = (JasmineEventNotification) jasmineEventNotificationObject;

                    JasmineEventNotification ev = new JasmineEventNotification(notification.getId(), notification.getDate(),
                        notification.getLevel(), notification.getMessage(), notification.getStatus(), notification.getRule());

                    List<JasmineEventNotificationDetail> details = new ArrayList<JasmineEventNotificationDetail>();
                    for (JasmineEventNotificationDetail detail : notification.getDetails()) {
                        JasmineEventNotificationDetail d = new JasmineEventNotificationDetail(detail.getLabel(),
                            detail.getMessage());
                        details.add(d);
                    }
                    ev.setDetails(details);

                    List<JasmineEventNotificationExecutedAction> actions = new ArrayList<JasmineEventNotificationExecutedAction>();
                    for (JasmineEventNotificationExecutedAction action : notification.getActions()) {
                        JasmineEventNotificationExecutedAction a = new JasmineEventNotificationExecutedAction(
                            action.getComment(), action.getAction(), action.getDate(), action.getUser());
                        actions.add(a);
                    }
                    ev.setActions(actions);

                    this.manager.add(ev);
                }
            }

            message.acknowledge();
        } catch (Throwable t) {
            logger.error("Failed processing JMS message", t);
        }
    }
}
